
/*********************************************************************************
*
*	CREADOR: Juan Manuel Fernndez Muoz
*
**********************************************************************************
**********************************************************************************
*
*	NOMBRE: fingerprint.h
*   
*
*	DESCRIPCIN:	Archivo de cabecera del algoritmo de reconocimiento de huellas
*					dactilares basado en correlacin de imgenes.
*
*********************************************************************************/

#ifndef _FINGERPRINT_H
#define _FINGERPRINT_H

// Definicin de constantes globales.
#define PI 3.14159265358979323846		// Constante que contiene un valor aproximado de la constante matemtica pi.
#define	NF	128							// Constante que indica el nmero de filas que tendrn las matrices complejas.
#define	NC	128							// Constante que indica el nmero de columnas que tendrn las matrices complejas.
#define	NFi	200							// Constante que indica el nmero de filas que tendrn las imgenes.
#define	NCi	152							// Constante que indica el nmero de columnas que tendrn las imgenes.
#define	NUMUSER	5						// Constante que indica el nmero de usuarios que puede haber registrados en la base de datos.

// Nuevo tipo que define los datos de cada usuario.
typedef struct
{
	unsigned char imageuser[NFi][NCi];	// Imagen de la huella del usuario.
	int userenroll;						// Variable que indica si el nmero de usario est registrado ya.
}user;

// Localizacin en memoria de las variables usadas.
#pragma DATA_SECTION (image, ".picture");
#pragma	DATA_SECTION (imagecapture, ".picture");
#pragma	DATA_SECTION (imagenroll, ".picture");
#pragma	DATA_SECTION (database, ".picture");
#pragma	DATA_SECTION (fftenroll, ".fft");
#pragma	DATA_SECTION (fftcapture, ".fft");
#pragma	DATA_SECTION (tw, ".fft");
#pragma DATA_SECTION (twi, ".fft");
#pragma DATA_SECTION (hist, ".info");
#pragma DATA_SECTION (simil, ".info");
#pragma DATA_SECTION (coord, ".info");
#pragma DATA_SECTION (phasepolar, ".info");

// Definicin de variables usadas.
unsigned char image[NFi][NCi];			// Imagen recivida del sensor.
unsigned char imagecapture[NFi][NCi];	// Copia de la imagen capturada con la que trabajaremos.
unsigned char imagenroll[NFi][NCi];		// Copia de la imagen de la base de datos con la que trabajaremos.
user database[NUMUSER];					// Definicin de una base de datos de NUMUSER miembros.
float fftenroll[NF][2*NC];				// Matriz compleja para la FFT 2D de la imagen de la base de datos.
float fftcapture[NF][2*NC];				// Matriz compleja para la FFT 2D de la imagen capturada.
float tw[NC];							// Tabla twiddle para la FFT 2D.
float twi[NC];							// Tabla twiddle para la IFFT 2D.
unsigned int hist[256];					// Vector que almacena el histograma de la imagen.
float simil;							// Varible que contendr el valor de comparacin de las imgenes.
int coord[2];							// Vector para almacenar el desplazamiento entre imgenes.
float phasepolar[2*NF];					// Tabla con valores de cosenos y senos necesarios para la transformacin a coordenadas polares.

// Variables globales usadas por el fichero "botones.gel".
unsigned int enrollFP=0, verifyFP=0, exitFP=0;

// Funciones necesarias para el control del fichero "botones.gel".
void dummyEnroll(void);
void dummyVerify(void);

// Funciones para tratar la imagen en el dominio espacial.
void gen_phase_table(float* , int);
void copy_image(unsigned char [][NCi], unsigned char [][NCi], int, int);
void histogram(unsigned char [][NCi], unsigned int*, int, int);
void preprocessing(unsigned char [][NCi], unsigned int*, int, int);
void complex_image(unsigned char [][NCi], float [][2*NC], int, int, int, int);

void desp_image(unsigned char [][NCi], int*, int, int, int);
void trans_polar_image(unsigned char [][NCi], float [][2*NC], int*, float*, int, int, int, int);

// Funciones para obtener la FFT 2D y la IFFT 2D.
void gen_twiddle(float*, int);
void copy_vector(float*, float*, int);
void complex_conj_vector(float*, int);
void bit_rev(float*, int);
void trasp_matrix(float [][2*NC], int, int);
#include "DSPF_sp_cfftr2_dit.h"
void fft2d(float [][2*NC], float*, float*, int, int);
void divide_ifft2d(float [][2*NC], int, int);

// Funciones para tratar la imagen en el dominio frecuencial.
void poc_function(float [][2*NC], float [][2*NC], int, int);
void poc(float [][2*NC], float [][2*NC], int, int);
float add_max_poc(float [][2*NC], int, int);
void high_peak_poc(float [][2*NC], int*, int, int);

#endif
