
/****************************************************************************
*
*              
*             TEXAS INSTRUMENTS PROPRIETARY INFORMATION
*
*  (c) Copyright, Texas Instruments Incorporated, 2003.
*      All Rights Reserved.
*
*  Property of Texas Instruments Incorporated. Restricted Rights -
*  Use, duplication, or disclosure is subject to restrictions set
*  forth in TI's program license agreement and associated documentation.
*
***************************************************************************** 
*****************************************************************************
*
* NAME: fpc1010.c
*   
*
* DESCRIPTION: file describing different function to read image from FPC1010 sensor.
*   
*
* DATE CREATED.. 04/12/2002 
* LAST MODIFIED. 06/24/2003
*   
*
* COMPILATION: CCS 2.20.05
*   
*
****************************************************************************/

/* include files */
#include <csl.h>
#include <csl_mcbsp.h>


#include "fpc1010.h"
#include "spi.h"

/*
 * Function name :  Sensor_init
 * Arguments     :  void 
 * Return value  :  void
 * Exceptions    : 
 * Description   :  Initialize the Sensor  
 */ 
    
void sensorInit(void)
{
	
	/* Hardware ware Reset  for 1 micor sec.*/
	
	SPI_RESET;
    SPI_SET;    
        
   	/* Initialization (part1) */
	SPI_ENABLE;
	SPI_write(RESET);
	SPI_DISABLE;
	SPI_ENABLE;
	SPI_write(SENSX);
	SPI_write(0x01);
	SPI_DISABLE;
	/* Initialization (part2) */
	SPI_ENABLE;
	SPI_write(SENSY);
	SPI_write(0x01);
	SPI_DISABLE;
	SPI_ENABLE;
	SPI_write(COMMAND);
	SPI_write(0x08);
	SPI_DISABLE;
}

/*
 * Function name :  Read_Image
 * Arguments     :  pointer, Address to store the image. 
 * Return value  :  void
 * Exceptions    : 
 * Description   :  Read the complete image 
 */
void readImage(unsigned char *image_ptr) 
{
	
	int numRows,numCols,i,j;
	unsigned char pix1, temp;
	unsigned volatile char dummy[6];
	
	unsigned char *temp_ptr;
	/* Read all 200 Rows */
	for(numRows=0;numRows<MAXROW  ;numRows++)
	{
    	temp_ptr = image_ptr;   
       	for(j=0;j<3;j++)
       	{
       		image_ptr = temp_ptr;
         	SPI_ENABLE;
	        SPI_write(READ_SHFTX);
            /* Ignore 32 clocks */
            for(i=0;i<5;i++) 
			    dummy[i]=SPI_read(); 
            /* Capture 152 8bit pixel in each row */    	
            for(numCols=0; numCols<MAXCOLUMN;numCols++)
            {
            	pix1 = SPI_read();
            	if(j==0)
            	{
	                *image_ptr++ = pix1;   
    			}
    			else if(j==1)
    			{
	                *image_ptr++ += pix1;   
    			}			
    			if(j==2)
    			{
    				temp = *image_ptr + pix1;
    				*image_ptr++ = ~temp;				
    			}	
    		}
    		
                    /* Ignore 16 clocks */
            for(i=0;i<2;i++) 
	            dummy[i]=SPI_read();
	        SPI_DISABLE;
	    }
        SPI_ENABLE;
        SPI_write(READ_SHFTY);
        SPI_DISABLE;           
	}       
}
