
/****************************************************************************
*
*              
*             TEXAS INSTRUMENTS PROPRIETARY INFORMATION
*
*  (c) Copyright, Texas Instruments Incorporated, 2003.
*      All Rights Reserved.
*
*  Property of Texas Instruments Incorporated. Restricted Rights -
*  Use, duplication, or disclosure is subject to restrictions set
*  forth in TI's program license agreement and associated documentation.
*
***************************************************************************** 
*****************************************************************************
*
* NAME: spi.c
*   
*
* DESCRIPTION:functions to read complete image from FPC1010 sensor. 
*   
*
* DATE CREATED.. 04/12/2002 
* LAST MODIFIED. 06/24/2003
*                
*   
*
* COMPILATION: CCS2.20.05 
*   
*
****************************************************************************/
/* Include files */
#include <csl.h>
#include <csl_mcbsp.h>


#include "spi.h"

/* Declar CSL object */
MCBSP_Handle hMcbsp0;   /* Handle for Mcbsp0 */

/* Declar Global variable */
extern volatile int SENSOR = TRUE;
/* Function Declaration */

/*
 * Function name :  ConfigMcbsp
 * Arguments     :  Integers, The SPI speed at which sensor can operate & swap condition.
 * Return value  :  Integer.
 * Exceptions    :  SPI speed must not be greater than 75MHz.
 * Description   :  Mcbsp Initialization.
 */
 
int ConfigMcbsp(int spiSpeed, int swap)
{
	
 	/* CLKDVD = [McBSP speed(recommned max)*SensorOperatingFrequency] -1 , clk speed for 6713=cpu speed/2 */
 	int clk_div = (((MCBSP_CLOCK_MHZ * 1000000) / spiSpeed) -1);

    MCBSP_Config mcbspCfg0 = {
        /* SPCR Setup */
        MCBSP_SPCR_RMK(
            MCBSP_SPCR_FREE_NO,             /* 0 */
            MCBSP_SPCR_SOFT_DEFAULT,        /* 0 */
            MCBSP_SPCR_FRST_DEFAULT,        /* 0 */
            MCBSP_SPCR_GRST_DEFAULT,        /* 0 */
            MCBSP_SPCR_XINTM_XRDY,          /* 00 */
            MCBSP_SPCR_XSYNCERR_DEFAULT,    /* 0 */
            MCBSP_SPCR_XRST_DEFAULT,        /* 0 */
            MCBSP_SPCR_DLB_OFF,             /* 0 */
            MCBSP_SPCR_RJUST_RZF,           /* 00 */
            MCBSP_SPCR_CLKSTP_DISABLE,      /* 0 */
            MCBSP_SPCR_DXENA_OFF,           /* 0 */
            MCBSP_SPCR_RINTM_RRDY,          /* 00 */
            MCBSP_SPCR_RSYNCERR_DEFAULT,    /* 0 */
            MCBSP_SPCR_RRST_DEFAULT         /* 0 */
        ),
        /* RCR Setup */
        MCBSP_RCR_RMK(
            MCBSP_RCR_RPHASE_SINGLE,        /* 0 */
            MCBSP_RCR_RFRLEN2_OF(0),        /* 0000 */
            MCBSP_RCR_RWDLEN2_DEFAULT,      /* 000 */
            MCBSP_RCR_RCOMPAND_OF(1),       /* 01 */
            MCBSP_RCR_RFIG_YES ,             /* 1 */
            MCBSP_RCR_RDATDLY_1BIT,         /* 01 */
            MCBSP_RCR_RFRLEN1_OF(0),        /* 00000 */
            MCBSP_RCR_RWDLEN1_8BIT,         /* 000 */
            MCBSP_RCR_RWDREVRS_DISABLE      /* 0 */
        ),
        /* XCR Setup */
        MCBSP_XCR_RMK(
            MCBSP_XCR_XPHASE_SINGLE,        /* 0 */
            MCBSP_XCR_XFRLEN2_OF(0),        /* 00000 */
            MCBSP_XCR_XWDLEN2_DEFAULT,      /* 000 */
            MCBSP_XCR_XCOMPAND_DEFAULT,     /* 00 */
            MCBSP_XCR_XFIG_YES,              /* 1 */
            MCBSP_XCR_XDATDLY_1BIT,         /* 01 */
            MCBSP_XCR_XFRLEN1_OF(0),        /* 00000 */
            MCBSP_XCR_XWDLEN1_8BIT,         /* 000 */
            MCBSP_XCR_XWDREVRS_DISABLE      /* 0 */
        ),
        /* SRGR Setup */
        MCBSP_SRGR_RMK(
            MCBSP_SRGR_GSYNC_FREE,          /* 0 */
            MCBSP_SRGR_CLKSP_RISING,        /* 0 */
            MCBSP_SRGR_CLKSM_INTERNAL,      /* 1 */
            MCBSP_SRGR_FSGM_DXR2XSR,        /* 0 */
            MCBSP_SRGR_FPER_DEFAULT,        /* 0 */
            MCBSP_SRGR_FWID_DEFAULT,        /* 0 */
            MCBSP_SRGR_CLKGDV_OF(0)        /* CLKGDV */
        ),
        /* RCER Setup */
        MCBSP_RCER_DEFAULT,
    
        
        /* XCER Setup */
        MCBSP_XCER_DEFAULT,
        /* MCR Setup */
        MCBSP_MCR_DEFAULT, /* default values */
        /* PCR Setup */
        MCBSP_PCR_RMK(
            MCBSP_PCR_XIOEN_SP,             /* 0 */
            MCBSP_PCR_RIOEN_SP,             /* 0 */
            MCBSP_PCR_FSXM_OF(1),           /* 1 */
            MCBSP_PCR_FSRM_OF(0),           /* 0 */
            MCBSP_PCR_CLKXM_OUTPUT,         /* 1 */
            MCBSP_PCR_CLKRM_INPUT ,         /* 0 */
            MCBSP_PCR_CLKSSTAT_0,           /* 0 */
            MCBSP_PCR_DXSTAT_0,             /* 0 */
            MCBSP_PCR_FSXP_ACTIVEHIGH,      /* 0 */
            MCBSP_PCR_FSRP_ACTIVEHIGH,      /* 0 */
            MCBSP_PCR_CLKXP_RISING,         /* 0 */
            MCBSP_PCR_CLKRP_FALLING         /* 0 */
        )
    };
    
    if(clk_div<0 || clk_div > 255)
    	return FALSE;
    mcbspCfg0.srgr = (mcbspCfg0.srgr & 0xFFFFFF00 ) | (clk_div & 0xFF);
    mcbspCfg0.xcr =  (mcbspCfg0.xcr & 0xFFE7FFFF )  | ((swap <<19)& 0x00180000);
    MCBSP_config(hMcbsp0, &mcbspCfg0);
    
    return TRUE;
} /* end of Config_McBSP(void) */


/*
 * Function name :  SPI_init
 * Arguments     :  Integer ,The first argument is Speed of communication
 *					2nd argument is for swapping MSB to LSB .
 * Return value  :  void
 * Exceptions    : 
 * Description   :  Initialize the SPI interface 
 */ 

int SPI_init(int spiSpeed,int swap)
{
	
	    int ret;
	    volatile int waittime;
	    if(!swap)
	    	SENSOR=FALSE;
	    /* Open the McBSP channel */
        hMcbsp0 = MCBSP_open(MCBSP_DEV0, MCBSP_OPEN_RESET);
        /*Setup for McBSP */
        ret= ConfigMcbsp(spiSpeed,swap);
        /* Start Sample Rate Generator: set /GRST = 1 */
        MCBSP_enableSrgr(hMcbsp0);
        /* inserted wait time for McBSP to get ready */
        for (waittime=0; waittime<2; waittime++);
        /* write desired value of CLKSTP in SPCR */
        MCBSP_FSET(SPCR0,CLKSTP,0x3);
        MCBSP_FSET(SPCR0,FRST,1); 
        /* Wake up the McBSP as transmitter and receiver */
        MCBSP_FSET(SPCR0,XRST,1);
        MCBSP_FSET(SPCR0,RRST,1);
        /* inserted wait time for McBSP to get ready */
        for (waittime=0; waittime<0x2; waittime++);
        return ret;
}

/*
 * Function name :  SPI_write
 * Arguments     :  integer , the commands to be written 
 * Return value  :  void
 * Exceptions    : 
 * Description   :  write the value in DXR.By the same time it's required to read some
 *					dummy byte from High Impedance state of Sensor Output.
 */
void SPI_write(int command)
{
	unsigned volatile char tmp;
	/* Checking  transmitter Ready bit */
	while(!(MCBSP_xrdy(hMcbsp0))){}
	/* Writing into DXR */
	MCBSP_write(hMcbsp0,command);
	/* Checking  Receiver Ready bit */
	while(!(MCBSP_rrdy(hMcbsp0))){}
	/* Store in a temp buffer which is not required */
	tmp=(unsigned char)MCBSP_read(hMcbsp0);
	
}

/*
 * Function name :  SPI_read
 * Arguments     :  void
 * Return value  :  character
 * Exceptions    : 
 * Description   :  write the value in DXR.
 */
unsigned char SPI_read(void)
{
	unsigned volatile char dummy;
	unsigned volatile char data;
	if(SENSOR)
		dummy=0xFF;
	else
		dummy=0x00;
	/* Checking  transmitter Ready bit */
	while(!(MCBSP_xrdy(hMcbsp0))){}
	/* Dummy write is required in order to send clock signal to sensor */
	MCBSP_write(hMcbsp0,dummy);
	/* Checking  receiver Ready bit */
	while(!(MCBSP_rrdy(hMcbsp0))){}
	data=(unsigned char)MCBSP_read(hMcbsp0);
	return data;
}
